/* eslint-disable new-cap */

const Collection    = require('./Collection');
const createElement = require('../utilities').createElement;
const View          = require('./View');

/**
 * A base view for lists / collections. This view is not a subclass of the View constructor.
 * @alias ListView
 * @memberof dlx.module:base
 * @extends dlx.module:base.View
 * @type {Function}
 */
class ListView extends View {
  /**
   * Create a new List View
   * @type {Object}
   * @param {Array}    [collection = []]     The Collection of models to create the List View from
   * @param {Object}   [options = {}]        An options object
   * @param {Function} [options.view = View] A View constructor to use for each item in the list
   */
  constructor(collection = [], options = {}) {

    const view = options.view || View;
    const opts = Object.assign({ view }, options);

    super(collection, opts);

    this.collection = new Collection(collection);
    this.view       = view;

  }

  /**
   * Compiles the List View template using the Collection data
   * @return {Object} Returns a UL element containing the collection of views
   */
  template() {

    const list = createElement('ul', { classList: 'list' });

    this.model.forEach(model => {

      const container = createElement('li');
      const view      = new this.view(model, this.options);

      view.render({ container });
      list.appendChild(container);

    });

    return list;

  }

}

module.exports = ListView;
