/**
 * The DLx object
 * @module dlx
 * @type {Object}
 * @prop {Object} base      Base models for other classes
 * @prop {Object} data      An object containing linguistic data, e.g. data about the IPA. See [dlx.data]{@link dlx.module:data}.
 * @prop {Object} models    Data models for linguistic objects, such as Text, Phrase, Word, etc. See [dlx.models]{@link dlx.module:models}.
 * @prop {Object} utilities A collection of utility functions. See [dlx.utils]{@link dlx.module:utils}.
 * @prop {Object} utils     Alias for dlx.utilities
 * @prop {Object} views     A collection of Views for various linguistic objects. See [dlx.views]{@link dlx.module:views}.
 */
const dlx = {
  base:      require('./base'),
  data:      require('./data'),
  models:    require('./models'),
  utilities: require('./utilities'),
  utils:     require('./utilities'),
  views:     require('./views'),
};

module.exports = dlx;
