/**
 * Data models for linguistic objects, such as Text, Phrase, Word, etc.
 * @memberof dlx
 * @module models
 * @type {Object}
 * @prop {Function} Access            A class representing a set of access protocols. See [Access]{@link dlx.module:models.Access}.
 * @prop {Function} Address           A class representing a postal address. See [Address]{@link dlx.module:models.Address}.
 * @prop {Function} Allomorph         A class representing an Allomorph of a Lexeme. See [Allomorph]{@link dlx.module:models.Allomorph}.
 * @prop {Function} Grapheme          A class representing a Grapheme. See [Grapheme]{@link dlx.module:models.Grapheme}.
 * @prop {Function} Language          A class representing an Language. See [Language]{@link dlx.module:models.Language}.
 * @prop {Function} Lexeme            A class representing a Lexeme. See [Lexeme]{@link dlx.module:models.Lexeme}.
 * @prop {Function} LexemeReference   A class representing a reference to a Lexeme. See [LexemeReference]{@link dlx.module:models.LexemeReference}.
 * @prop {Function} LexicalRelation   A class representing a lexical relation.
 * @prop {Function} Location          A class representing a location. See [Location]{@link dlx.module:models.Location}.
 * @prop {Function} Morpheme          A class representing a token of a morpheme in a corpus. See [Morpheme]{@link dlx.module:models.LexemeReference}.
 * @prop {Function} MultiLangString   A class representing a Multi-Language String. See [MultiLangString]{@link dlx.module:models.MultiLangString}.
 * @prop {Function} Note              A class representing a Note. See [Note]{@link dlx.module:models.Note}.
 * @prop {Function} Orthography       A class representing an Orthography. See [Orthography]{@link dlx.module:models.Orthography}.
 * @prop {Function} Phone             A class representing a Phoneme Token. See [Phone]{@link dlx.module:models.Phone}.
 * @prop {Function} Phoneme           A class representing a Phoneme, as an abstract type. See [Phoneme]{@link dlx.module:models.Phoneme}.
 * @prop {Function} Phrase            A class representing a Phrase. See [Phrase]{@link dlx.module:models.Phrase}.
 * @prop {Function} Place             An alias for [Location]{@link dlx.module:models.Location}.
 * @prop {Function} Reference         A class representing a Bibliographic Reference. See [Reference]{@link dlx.module:models.Reference}.
 * @prop {Function} Sense             A class representing a Sense. See [Sense]{@link dlx.module:models.Sense}.
 * @prop {Function} String            An alias for [MultiLangString]{@link dlx.module:models.MultiLangString}.
 * @prop {Function} Tags              A class representing a set of tags, as a Map object. See [Tags]{@link dlx.module:models.Tags}.
 * @prop {Function} Variant           A class representing a lexeme variant reference.
 * @prop {Function} Word              A class representing a Word. See [Word]{@link dlx.module:models.Word}.
 */

module.exports = {
  // NB: The require statements here must match the filenames exactly in order for the tests to work
  Access:          require('./Access'),
  Address:         require('./Address'),
  Allomorph:       require('./Allomorph'),
  Grapheme:        require('./Grapheme'),
  Language:        require('./Language'),
  Lexeme:          require('./Lexeme'),
  LexemeReference: require('./LexemeReference'),
  LexicalRelation: require('./LexicalRelation'),
  Location:        require('./Location'),
  Morpheme:        require('./Morpheme'),
  MultiLangString: require('./MultiLangString'),
  Note:            require('./Note'),
  Orthography:     require('./Orthography'),
  Phone:           require('./Phone'),
  Phoneme:         require('./Phoneme'),
  Phrase:          require('./Phrase'),
  Place:           require('./Location'),
  Reference:       require('./Reference'),
  Sense:           require('./Sense'),
  String:          require('./MultiLangString'),
  Tags:            require('./Tags'),
  Variant:         require('./Variant'),
  Word:            require('./Word'),
};
