/* global dlx */

/* eslint-disable
  func-names,
  max-nested-callbacks,
  max-statements,
  no-empty-function,
  no-invalid-this,
  no-magic-numbers,
  no-shadow,
  object-curly-newline,
  object-property-newline,
  prefer-arrow-callback
*/

describe('dlx.base.Emitter', function() {

  const Emitter = dlx.base.Emitter;

  it('Errors', function() {
    expect(() => new Emitter()).not.toThrow();
  });

  it('new Emitter()', function() {
    expect(() => new Emitter()).not.toThrow();
    class MyClass extends Emitter {}
    const myClass = new MyClass;
    expect(myClass instanceof MyClass).toBe(true);
    expect(myClass instanceof Emitter).toBe(true);
  });

  it('Emitter.extend()', function() {

    const extend = () => Emitter.extend(Array);
    expect(extend).not.toThrow();

    const MyClass = Emitter.extend(Array);
    const myClass = new MyClass;
    expect(myClass instanceof Array).toBe(true);
    expect(myClass instanceof MyClass).toBe(true);
    expect(myClass instanceof Emitter).toBe(false);

  });

  it('Emitter.prototype.events', function() {
    const emitter = new Emitter;
    expect(emitter.events instanceof Map).toBe(true);
  });

  it('Emitter.prototype.emit()', function() {

    const emitter = new Emitter;
    const cb1 = () => {};
    const cb2 = () => {};
    const callbacks = { cb1, cb2 };

    spyOn(callbacks, 'cb1');
    spyOn(callbacks, 'cb2');

    emitter.events.set('myEvent', new Set);
    emitter.events.get('myEvent').add(callbacks.cb1);
    emitter.events.get('myEvent').add(callbacks.cb2);

    expect(() => emitter.emit('myEvent')).not.toThrow();
    expect(callbacks.cb1).toHaveBeenCalled();
    expect(callbacks.cb2).toHaveBeenCalled();

  });

  it('Emitter.prototype.off()', function() {

    const emitter = new Emitter;
    const obj = {};
    const cb1 = function() { expect(this).toBe(obj); }.bind(obj);
    const cb2 = () => {};
    const cb3 = () => {};
    const cb4 = () => {};

    emitter.on({
      cb1,
      cb2,
      cb3,
      cb4,
    });

    emitter.off('cb1', cb1);
    expect(emitter.events.size).toBe(3);

  });

  it('Emitter.prototype.on()', function() {

    const emitter = new Emitter;

    expect(emitter.on).toThrow();
    expect(() => emitter.on(true, function() {})).not.toThrow();
    expect(() => emitter.on('update')).toThrow();
    expect(() => emitter.on('update', true)).toThrow();
    expect(() => emitter.on('update', function() {})).not.toThrow();

    const e       = new Emitter;
    const obj     = { hello: 'world' };
    const boundCb = function() { expect(this).toBe(obj); }.bind(obj);

    const callbacks = {
      boundCb,
      cb() {},
      cb1() {},
      cb2() {},
    };

    spyOn(callbacks, 'cb');
    spyOn(callbacks, 'boundCb').and.callThrough();

    e.on('update', callbacks.cb);
    e.on('update', callbacks.boundCb, obj);
    expect(e.events.get('update').has(callbacks.cb)).toBe(true);
    e.events.get('update').forEach(call => call());

    const events = {
      delete: callbacks.cb1,
      save:   callbacks.cb2,
    };

    e.on(events);

    expect(e.events.get('delete')).toBeDefined();
    expect(e.events.get('save').has(callbacks.cb2)).toBe(true);

  });

  it('Emitter.prototype.on({Promise})', function(done) {
    const emitter = new Emitter;
    const asyncCallback = () => new Promise(resolve => {
      setTimeout(resolve, 500);
    });
    emitter.on('event', asyncCallback);
    const emit = () => emitter.emit('event').then(done).catch(fail);
    expect(emit).not.toThrow();
  });

  it('Emitter.prototype.once()', function() {

    const emitter = new Emitter;
    const cb1     = () => {};

    expect(emitter.once).toThrow();
    expect(() => emitter.once('eventName')).toThrow();
    expect(() => emitter.once('cb1', cb1)).not.toThrow();
    expect(emitter.events.size).toBe(1);
    expect(emitter.events.get('cb1').size).toBe(1);
    expect(emitter.events.get('cb1').has(cb1)).toBe(false);

    const runCallback = () => emitter.events.get('cb1').forEach(cb => cb());

    expect(runCallback).not.toThrow();
    expect(emitter.events.size).toBe(0);

  });

});
