/* global dlx */

/* eslint-disable
  func-names,
  init-declarations,
  max-nested-callbacks,
  max-statements,
  no-magic-numbers,
  no-shadow,
  object-curly-newline,
  object-property-newline,
  prefer-arrow-callback
*/

describe('dlx.models.Language', function() {

  const { Model }    = dlx.base;
  const { Language } = dlx.models;

  const data = {
    abbreviation:    'ctm',
    additionalNames: ['Shetimasha'],
    customProperty:  'customValue',
    glottolog:       'chit1248',
    iso:             'ctm',
    locations:       [{ region: 'Louisiana' }],
    name:            { eng: 'Chitimacha' },
    phonemes:        [{}],
  };

  let lang;

  it('Errors', function() {
    expect(() => new Language(true)).toThrow();
    expect(() => new Language).not.toThrow();
    expect(() => { lang = new Language(data); }).not.toThrow();
  });

  it('new Language()', function() {
    expect(lang instanceof Model).toBe(true);
  });

  it('Language.prototype.additionalNames', function() {
    expect(lang.additionalNames.has(data.additionalNames[0])).toBe(true);
    expect(() => lang.additionalNames.push(true)).toThrow();
    expect(new Language().toJSON().additionalNames).toBeUndefined();
  });

  it('Language.prototype.glottolog', function() {
    expect(() => { lang.glottolog = 'abcd1234'; }).not.toThrow();
    expect(() => { lang.glottolog = 'abc'; }).toThrow();
  });

  it('Language.prototype.iso', function() {
    expect(() => { lang.iso = 'ctm'; }).not.toThrow();
    expect(() => { lang.iso = 'en'; }).toThrow();
  });

  it('Language.prototype.dateCreated', function() {
    expect(lang.dateCreated instanceof Date).toBe(true);
    expect(lang.toJSON().dateCreated instanceof Date).toBe(true);
  });

  it('Language.prototype.dateModified', function() {
    expect(lang.dateCreated instanceof Date).toBe(true);
    expect(lang.toJSON().dateModified instanceof Date).toBe(true);
    const originalDate = lang.dateModified;
    lang.jambo = 'dunia';
    expect(lang.dateModified).not.toBe(originalDate);
  });

});
