/* global dlx */

/* eslint-disable
  func-names,
  max-nested-callbacks,
  max-statements,
  no-magic-numbers,
  no-shadow,
  object-curly-newline,
  object-property-newline,
  prefer-arrow-callback
*/

describe('dlx.base.Model', function() {

  const { Emitter, Model } = dlx.base;
  const { MultiLangString } = dlx.models;

  const model = new Model;

  it('new Model()', function() {
    expect(() => new Model).not.toThrow();
    expect(model instanceof Emitter).toBe(true);
    expect(model.destroy instanceof Function).toBe(true);
    expect(model.save instanceof Function).toBe(true);
  });

  it('Model.defineEnumProp()', function() {
    const o = {};
    const vals = ['hello', 'hola', 'jambo'];
    expect(() => Model.defineEnumProp(o, 'arr1', vals, 'bad')).toThrow();
    expect(() => Model.defineEnumProp(o, 'arr2', vals, 'hello')).not.toThrow();
    expect(() => { o.arr2 = 'bad'; }).toThrow();
  });

  it('Model.defineProp()', function() {
    const o = {};
    const data = { eng: 'hello' };
    expect(() => Model.defineProp(o, 'transcription', MultiLangString, data)).not.toThrow();
    expect(o.transcription instanceof MultiLangString).toBe(true);
    expect(o.transcription.eng).toBe(data.eng);
  });

  it('Model.defineArray()', function() {
    const o = {};
    const arr = ['test'];
    class MyClass { constructor(data) { this.data = data; } }
    expect(() => Model.defineArray(o, 'myClass', MyClass, arr)).not.toThrow();
    expect(o.myClass[0] instanceof MyClass).toBe(true);
    expect(o.myClass[0].data).toBe('test');
  });

  it('Model.simplify()', function() {

    const obj = {
      emptyArr:      [],
      emptyMap:      new Map,
      emptyObj:      {},
      emptySet:      new Set,
      emptyStr:      '',
      nullProp:      null,
      undefinedProp: undefined,

      filledArr:     ['hello'],
      filledMap:     new Map([['hello', 'world']]),
      filledObj:     { hello: 'world' },
      filledSet:     new Set(['hello']),
      numProp:       0,
      booleanProp:   true,

      required:      [],
    };

    const o = Model.simplify(obj, ['required']);

    expect(o).not.toBe(obj);

    expect(o.emptyArr).toBeUndefined();
    expect(o.emptyMap).toBeUndefined();
    expect(o.emptyObj).toBeUndefined();
    expect(o.emptySet).toBeUndefined();
    expect(o.emptyStr).toBeUndefined();
    expect(o.nullProp).toBeUndefined();
    expect(o.undefinedProp).toBeUndefined();

    expect(o.filledArr).toBe(obj.filledArr);
    expect(o.filledMap).toBe(obj.filledMap);
    expect(o.filledObj).toBe(obj.filledObj);
    expect(o.filledSet).toBe(obj.filledSet);
    expect(o.numProp).toBe(obj.numProp);
    expect(o.booleanProp).toBe(obj.booleanProp);

    expect(o.required).toBe(obj.required);

  });

});
