/* global dlx */

/* eslint-disable
  func-names,
  global-require,
  max-nested-callbacks,
  no-global-assign,
  no-magic-numbers,
  no-shadow,
  object-curly-newline,
  object-property-newline,
  prefer-arrow-callback
*/

describe('dlx.models.Allomorph', function() {

  const { Allomorph, MultiLangString } = dlx.models;

  const allomorph = new Allomorph;

  it('Errors', function() {

    const badData = () => new Allomorph('dut');
    const badEnvs = () => new Allomorph({ environments: '_C', transcription: {} });
    const badTrx  = () => new Allomorph({ environments: [], transcription: { test: 42 } });

    expect(badData).toThrow();
    expect(badEnvs).toThrow();
    expect(badTrx).toThrow();

  });

  it('new Allomorph()', function() {

    expect(allomorph.environments instanceof Set).toBe(true);
    expect(allomorph.transcription instanceof MultiLangString).toBe(true);

    const oldAllmorph = new Allomorph;
    const newAllomorph = new Allomorph(oldAllmorph);
    expect(oldAllmorph).toBe(newAllomorph);

  });

  it('Allomorph.prototype.{customProperty}', function() {

    const allomorph = new Allomorph({
      customProperty1: 'custom value 1',
      environments:    [],
      transcription:   { spa: 'hola' },
    });

    allomorph.customProperty2 = 'custom value 2';

    expect(allomorph.customProperty1).toBe('custom value 1');
    expect(allomorph.customProperty2).toBe('custom value 2');

  });

  it('Allomorph.prototype.environments', function() {
    const envs = allomorph.environments;
    expect(() => { allomorph.environments = []; }).not.toThrow();
    expect(allomorph.environments).toBe(envs);
    expect(() => { allomorph.environments.add(true); }).toThrow();
    expect(() => { allomorph.environments.add('_C'); }).not.toThrow();
    expect(allomorph.environments.size).toBe(1);
  });

  it('Allomorph.prototype.toJSON()', function() {

    const json = JSON.stringify(allomorph);
    const pojo = JSON.parse(json);

    expect(Array.isArray(pojo.environments)).toBe(true);
    expect(typeof pojo.transcription).toBe('object');

  });

});
