/* global dlx */

/* eslint-disable
  func-names,
  max-nested-callbacks,
  max-statements,
  no-empty-function,
  no-magic-numbers,
  no-shadow,
  object-curly-newline,
  object-property-newline,
  prefer-arrow-callback
*/

describe('DOMListener', function() {

  const { DOMListener } = dlx.base;
  const el = document.getElementById('Test');

  it('Errors', function() {
    expect(() => new DOMListener).toThrow();
    expect(() => new DOMListener(el)).not.toThrow();
    expect(() => new DOMListener('#Test')).not.toThrow();
  });

  it('DOMListener.prototype.off()', function() {

    const listener = new DOMListener(el);

    listener.events.clear();

    const cb1 = () => {};

    listener.events.set('click', new Set([cb1]));

    expect(() => listener.off('click', cb1)).not.toThrow();
    expect(listener.events.size).toBe(0);

    const cb2 = () => {};
    const cb3 = () => {};

    listener.events.set('change', new Set([cb2, cb3]));

    expect(() => listener.off('change')).not.toThrow();
    expect(listener.events.size).toBe(0);

    listener.events.set('select', new Set([cb2]));
    listener.events.set('drag', new Set([cb3]));

    expect(() => listener.off()).not.toThrow();
    expect(listener.events.size).toBe(0);

  });

  it('DOMListener.prototype.on()', function() {

    const listener = new DOMListener(el);

    listener.events.clear();

    const cb1 = () => {};

    expect(() => listener.on('click', cb1)).not.toThrow();
    expect(listener.events.get('click') instanceof Set).toBe(true);
    expect(listener.events.get('click').has(cb1)).toBe(true);

  });

  it('DOMListener.prototype.once()', function() {

    const listener = new DOMListener(el);

    listener.events.clear();

    const cb1 = () => {};
    const callbacks = { cb1 };
    spyOn(callbacks, 'cb1');

    expect(() => listener.once('click', callbacks.cb1)).not.toThrow();
    expect(listener.events.get('click') instanceof Set).toBe(true);

    el.dispatchEvent(new Event('click'));
    expect(callbacks.cb1).toHaveBeenCalled();

  });

});
