/* global dlx */

/* eslint-disable
  func-names,
  max-nested-callbacks,
  max-statements,
  no-empty-function,
  no-magic-numbers,
  no-shadow,
  object-curly-newline,
  object-property-newline,
  prefer-arrow-callback
*/

describe('View', function() {

  const { Emitter, View } = dlx.base;

  const data = {};
  const options = { el: '#Test .container', position: 'start' };
  const view    = new View(data, options);

  beforeAll(function() {
    view.el.innerHTML = '';
  });

  afterAll(function() {
    view.el.innerHTML = '';
  });

  it('Errors', function() {
    expect(() => new View).not.toThrow();
    expect(() => new View(data)).not.toThrow();
  });

  it('new View()', function() {
    expect(view instanceof Emitter).toBe(true);
  });

  it('View.prototype.el', function() {

    expect(view.el instanceof Node).toBe(true);

    const genericEl = new View(data);
    expect(genericEl.el.tagName).toBe('DIV');

    const el = document.createElement('div');
    const v  = new View(data, { el });
    expect(v.el).toBe(el);

  });

  it('View.prototype.position', function() {
    expect(view.position).toBe(options.position);
    const noPosition = new View(data);
    expect(noPosition.position).toBe('end');
  });

  it('View.prototype.template', function() {
    expect(typeof view.template).toBe('function');
  });

  it('View.prototype.render()', function() {
    const view = new View(data);
    expect(() => view.render()).toThrow();
    expect(() => view.render({ container: options.el })).not.toThrow();
    expect(() => view.render({ container: options.el, position: 'replace' })).not.toThrow();
  });

  it('View.prototype.clear()', function() {
    const view = new View(data, { container: options.el, el: document.createElement('div') });
    expect(() => view.clear()).not.toThrow();
    expect(document.querySelector(options.el).children.length).toBe(0);
  });

  it('View.prototype.remove()', function() {
    const el = document.createElement('div');
    el.id = 'ID';
    const view = new View(data, { el });
    expect(() => view.remove()).not.toThrow();
    expect(document.getElementById(el.id)).toBe(null);
  });

});
